/**************************************************************************************************
* NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
*
* Copyright 2010 Flexera Software, Inc. and/or InstallShield Co. Inc.  All Rights Reserved.  
*
* This computer program is the property of Flexera Software, Inc. and InstallShield Co. Inc. of 
* Schaumburg, Illinois, U.S.A.  Any use, copy, publication, distribution, display, modification, 
* or transmission of this computer program in whole or in part in any form or by any means without
* the prior express written permission of Flexera Software, Inc. or InstallShield Co. Inc. is 
* strictly prohibited.
* 
* Except when expressly provided by Flexera Software, Inc. or InstallShield Co. Inc. in writing,
* possession of this computer program shall not be construed to confer any license or rights under 
* any of Flexera Software, Inc.s or InstallShield Co. Inc.s intellectual property rights, whether
* by estoppel, implication, or otherwise.  
* 
* ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
**************************************************************************************************/
/*
	This header file is part of an FNP sample utility that demonstrates Composite Transactions.

	It declares helper functions and macros for printing, memory allocation, string handling etc.
*/
#ifndef COMP_TRAN_UTIL_HELPER_H
#define COMP_TRAN_UTIL_HELPER_H

#include <stdlib.h>
#include <stdarg.h>

#include "FlxActTypes.h"

/*
	Printing.
	Output, e.g. response XML, is written to stdout.
	Informative and error messages are written to stderr.
*/
void flxCtuPrintOutput(const char* pszFormatStr, ...);
void flxCtuPrintOutputV(const char* pszFormatStr, va_list argList);
void flxCtuSetBrief(FlxActBool bIsBrief); /* Brief = don't print messages. */
void flxCtuPrintMessage(const char* pszFormatStr, ...);
void flxCtuPrintMessageV(const char* pszFormatStr, va_list argList);
void flxCtuPrintError(const char* pszFormatStr, ...);
void flxCtuPrintErrorV(const char* pszFormatStr, va_list argList);

void flxCtuSetVerbose(FlxActBool bIsVerbose);
void flxCtuPrintVerbose(const char *pszFormatStr, ...);
void flxCtuPrintVerboseV(const char *pszFormatStr, va_list argList);

/* 
	Confirmation. 
*/
FlxActBool flxCtuConfirm(const char *pszFormatStr, ...);

/* 
	Memory allocation - these functions exit on no memory rather than fail, to avoid cluttering 
	the sample with checks.  Realloc and free also do the right thing if passed NULL.
*/
void* flxCtuMalloc(size_t len);
void* flxCtuMallocAndZeroise(size_t len);
void* flxCtuRealloc(const void *pMemory, size_t len);
void  flxCtuFree(const void *pMemory);

/* 
	Function called if no memory is available - it must be implemented by users of this module.
*/
void flxCtuOnNoMemory();

/*
	String validation.
*/
FlxActBool flxCtuStrIsNumeric(const char* pSource);

/*
	String replace - destination string freed and replaced with duplicate of source string.
*/
void flxCtuStrNDupReplace(const char** ppszDestination, const char* pSource, size_t len);
void flxCtuStrDupReplace( const char** ppszDestination, const char* pszSource);

/*
	String duplicate.
*/
char* flxCtuStrNDup(const char* pString, size_t len);
char* flxCtuStrDup( const char* pszString);
char* flxCtuStrDupDelim(const char** ppszString, char delimiter);

/*
	String convert to lower case.
*/
char* flxCtuStrNLwr(char* pString, size_t len);
char* flxCtuStrLwr (char* pszString);

/*
	String duplicate and convert to lower case.
*/
char* flxCtuStrNDupLwr(const char* pString, size_t len);
char* flxCtuStrDupLwr (const char* pszString);

/*
	String compare ignoring case.
	Lower case duplicates of the strings are compared with strcmp.
*/
int flxCtuStrCmpLwr(const char* pszString1, const char* pszString2);

/*
	Extract and return a duplicate of the filename from a path.
*/
char* flxCtuStrDupNameFromPath(const char* pszPath);

/*
	Duplicate and return the sub-strings before and after the separator.
	If separator missing, before string will be a copy, after string will be empty.
*/
void flxCtuStrDupSeparate(const char*	pszString, 
						  char			separator, 
						  const char**	ppDupBefore,
						  const char**	ppDupAfter);

/*
	Assert.
	The target function flxCtuOnAssert must be implemented by users of this module.
*/
void flxCtuOnAssert(int line, const char* pszFilename);
#define FLX_CTU_ASSERT(condition) if (!(condition)) flxCtuOnAssert(__LINE__, __FILE__);
#endif	/* Include guard */
